import React, { FC, useCallback, useState } from 'react';
import { VirtualizedListItemsModel } from '../listItemsModels';
import { RefreshControl, View, VirtualizedList } from 'react-native';
import { LabelMediumText } from '../../../atoms';
import { styles } from './style';

const wait = (timeout: number) => {
  // @ts-ignore
  return new Promise(resolve => setTimeout(resolve, timeout));
};

export const VirtualizedListItemsWrapper: FC<VirtualizedListItemsModel> = ({
  listItemsValues,
  ItemComponent,
  wrapperTitle,
  onClick,
  scrollEnabled= true,
  listKey,
  initialScrollIndex = 0,
  onScreenRefresh,
}) => {
  const [refreshing, setRefreshing] = useState(false);

  const onRefresh = useCallback(() => {
    onScreenRefresh && onScreenRefresh();
    setRefreshing(true);
    wait(3000).then(() => setRefreshing(false));
  }, []);

  return (
    <>
      {ItemComponent && listItemsValues &&
        <VirtualizedList
          refreshControl={
            <RefreshControl
              refreshing={refreshing}
              onRefresh={onRefresh}
            />
          }
          onRefresh={onRefresh}
          listKey={listKey}
          showsVerticalScrollIndicator={false}
          initialScrollIndex={initialScrollIndex}
          scrollEnabled={scrollEnabled}
          data={listItemsValues}
          getItem={(data, index) => data[index]}
          getItemCount={(data) => data.length}
          keyExtractor={(item, index) => index.toString()}
          renderItem={({ item , index }) => {
            if (item) {
              // @ts-ignore
              const { id } = item;
              // @ts-ignore
              const RenderComponent = item.Component || ItemComponent;
              const isVirtualizedComponent = RenderComponent === VirtualizedListItemsWrapper;
              const VirtualizedComponent = <VirtualizedListItemsWrapper {...item} />;
              const CustomComponent = <RenderComponent {...item} onClick={() => id && onClick && onClick(id)} />;

              const SetterComponent = isVirtualizedComponent ? VirtualizedComponent : CustomComponent;
              return (
                <View key={index} style={styles.listItemsWrapperRow}>
                  {SetterComponent}
                </View>
              );
            }
            return (
              <View key={index} style={styles.listItemsWrapperRow}>
                <ItemComponent />
              </View>
            );
          }}
          ListHeaderComponent={() => (
            <>
              {wrapperTitle &&
                <View style={styles.listItemsWrapperTitle}>
                  <LabelMediumText color={styles.listItemsWrapperTitle.color}>{wrapperTitle}</LabelMediumText>
                </View>
              }
            </>
          )}
        />
      }
    </>
  );
};
